@@ -1,7 +1,10 @@
 #!/usr/bin/python3
 # -*- coding: utf-8; -*-
 
-from test.support import EnvironmentVarGuard
+try:
+    from test.support import EnvironmentVarGuard
+except ImportError:
+    from test.support.os_helper import EnvironmentVarGuard
 import unittest
 
 import debconf
@@ -21,7 +24,7 @@ class TimezoneTests(unittest.TestCase):
         controller.dbfilter = self.ubi_timezone.Page(None, db=db)
         self.gtk = self.ubi_timezone.PageGtk(controller)
 
-    @mock.patch('gi.repository.Soup.SessionAsync')
+    @mock.patch('gi.repository.Soup.Session')
     @mock.patch('gi.repository.Soup.Message')
     @mock.patch('json.loads')
     def test_city_entry(self, json_mock, *args):
