@@ -27,10 +27,9 @@ import syslog
 
 import debconf
 
+from ubiquity import i18n, plugin_manager
 from ubiquity.debconfcommunicator import DebconfCommunicator
 from ubiquity.misc import drop_privileges, execute_root
-from ubiquity import i18n
-from ubiquity import plugin_manager
 
 
 # Lots of intentionally unused arguments here (abstract methods).
@@ -476,3 +475,6 @@ class BaseFrontend:
 
     def set_online_state(self, state):
         pass
+
+    def save_oem_metapackages_list(self, wait_finished=False):
+        pass
